package com.github.droidworksstudio.mlauncher.data

import android.content.Context
import android.util.Log
import com.github.droidworksstudio.mlauncher.BuildConfig

class Migration(val context: Context) {
    fun migratePreferencesOnVersionUpdate(prefs: Prefs) {
        val currentVersionCode = BuildConfig.VERSION_CODE
        val savedVersionCode = prefs.appVersion

        // Define a map of version code -> preferences to clear
        val versionCleanupMap = mapOf(
            171 to listOf(
                "APP_DARK_COLORS",
                "APP_LIGHT_COLORS",
                "HOME_FOLLOW_ACCENT",
                "ALL_APPS_TEXT"
            ),
            172 to listOf(
                "TIME_ALIGNMENT",
                "SHOW_TIME",
                "SHOW_TIME_FORMAT",
                "TIME_COLOR"
            ),
            175 to listOf(
                "CLICK_APP_USAGE"
            ),
            10803 to listOf(
                "SHOW_EDGE_PANEL",
                "EDGE_APPS_NUM"
            ),
            10812 to listOf(
                "LOCK_MODE"
            ),
            1100504 to listOf(
                "SHOW_AZSIDEBAR"
            ),
            1100508 to listOf(
                "EXPERIMENTAL_OPTIONS"
            )
            // Add more versions and preferences to remove here
        )

        // Iterate over the versions and clear the relevant preferences
        for ((version, keys) in versionCleanupMap) {
            // Only clear preferences for versions between savedVersionCode and currentVersionCode
            if (version in (savedVersionCode + 1)..currentVersionCode) {
                // Remove the preferences for this version
                keys.forEach { key ->
                    prefs.remove(key)
                }
            }
        }

        // Update the stored version code after cleanup
        prefs.appVersion = currentVersionCode
    }

    fun migrateMessages(prefs: Prefs) {
        try {
            // Try to parse as the new format
            val messages = prefs.loadMessages()
            if (messages.isNotEmpty()) {
                // Already in correct format — no need to migrate
                return
            }
        } catch (_: Exception) {
            // If loading as new format fails, try migrating from the old format
            try {
                val wrongMessages = prefs.loadMessagesWrong()
                val correctedMessages = wrongMessages.map { wrong ->
                    Message(
                        text = wrong.a,
                        timestamp = wrong.b,
                        category = wrong.c,
                        priority = wrong.d
                    )
                }
                prefs.saveMessages(correctedMessages)
                Log.d("Migration", "Migration passed")
            } catch (e: Exception) {
                // Log or handle if even legacy format is broken
                Log.e("Migration", "Migration failed", e)
            }
        }
    }
}